﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using NLScC.model;
using NLScC.Command;
using System.Windows;
using System.Diagnostics;

namespace NLScC.ViewModel
{
    public class AutostartViewModel
    {

        #region Commands
        public ICommand AutostartNewAppCommand { get; internal set; }
        public ICommand AutostartNewWebCommand { get; internal set; }
        public ICommand AutostartNewFolderCommand { get; internal set; }
        public ICommand AutostartRemoveSelectedCommand { get; internal set; }
        public ICommand AutostartRemoveAllCommand { get; internal set; }
        public ICommand AutostartSaveConfigCommand { get; internal set; }
        public ICommand AutostartLoadConfigCommand { get; internal set; }
        public ICommand AutostartExecuteSelectedCommand { get; internal set; }
        public ICommand AutostartExecuteAllComand { get; internal set; }

        private void InitializeCommands()
        {
            AutostartNewAppCommand = new RelayCommand(AddNewAppExecute, CanAlwaysExecute);
            AutostartNewWebCommand = new RelayCommand(AddNewWebExecute, CanAlwaysExecute);
            AutostartNewFolderCommand = new RelayCommand(AddNewFolderExecute, CanAlwaysExecute);
            AutostartRemoveAllCommand = new RelayCommand(RemoveAllExecute, CanExecuteWithEntriesOnly);
            AutostartRemoveSelectedCommand = new RelayCommand(RemoveSelectedExecute, CanExecuteWithEntriesOnly);
            AutostartSaveConfigCommand = new RelayCommand(SaveConfigExecute, CanExecuteWithEntriesOnly);
            AutostartLoadConfigCommand = new RelayCommand(LoadCnfigExecute, CanAlwaysExecute);
            AutostartExecuteAllComand = new RelayCommand(ExecuteAllExecute, CanExecuteWithEntriesOnly);
            AutostartExecuteSelectedCommand = new RelayCommand(ExecuteSelectedExecute, CanExecuteWithEntriesOnly);
        }


        private bool CanAlwaysExecute(object obj)
        {
            return true;
        }

        private bool CanExecuteWithEntriesOnly(object obj)
        {
            return AutostartView.Count > 0;
        }

        private void ExecuteSelectedExecute(object obj)
        {

            ManagedApplication managed;
            if (obj == null) managed = _autostartView.CurrentItem as ManagedApplication;
            else managed = obj as ManagedApplication;
            if (managed != null)
            {
                switch (managed.AppType)
                {
                    case ApplicationType.Application:
                        Executioner.ExecuteApp(managed.Path, managed.Args);
                        break;
                    case ApplicationType.Chrome:
                    case ApplicationType.Firefox:
                    case ApplicationType.Edge:
                    case ApplicationType.Website:
                        Executioner.ExecuteWebsite(managed.Path, managed.AppType);
                        break;
                    case ApplicationType.Folder:
                        Executioner.ExecuteFolder(managed.Path, managed.Args);
                        break;
                    default:
                        LogConsole.Log("Der ausgewählte Eintrag <" + managed.Path + "> konnte nicht gestartet werden {ApplicationType->Unknown}", LogConsole.LogLevel.Warning);
                        break;
                }
            }
        }

        private void ExecuteAllExecute(object obj)
        {
            foreach (ManagedApplication app in _autostarts)
            {
                ExecuteSelectedExecute(app);
            }
        }

        private void LoadCnfigExecute(object obj)
        {
            System.Windows.Forms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog()
            {
                Filter = AppSettings.AutostartFilter,
                AddExtension = true,
                InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory),
                Title = "Autostart-Konfiguration laden"
            };
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                ManagedAppCollection o = (Reader.ReadFile(dialog.FileName, _autostarts, "app") as ManagedAppCollection);
                NLScC.dialog.OverrideDialog.OverrideMode mode = NLScC.dialog.OverrideDialog.OverrideMode.Override;
                if (o.Count > 0 && _autostarts.Count > 0)
                {
                    NLScC.dialog.OverrideDialog overrideDialog = new dialog.OverrideDialog()
                    {
                        Top = MainWindow.AppReference.Top + 10,
                        Left = MainWindow.AppReference.Left + (MainWindow.AppReference.Width / 2),
                        Owner = MainWindow.AppReference
                    };
                    if (overrideDialog.ShowDialog() == true)
                    {
                        mode = overrideDialog.OverrideResult;
                    }
                }
                if (mode == NLScC.dialog.OverrideDialog.OverrideMode.Override) _autostarts.Clear();
                LogConsole.Log("Konfiguration geladen -> OverrideMode." + mode);
                CopyToCollection(o);
            }
        }

        private void CopyToCollection(ManagedAppCollection collection)
        {
            foreach (ManagedApplication ma in collection)
            {
                _autostarts.Add(ma);
            }
            LogConsole.Log("Konfigruation erfolgreich geladen");
        }

        private string _path = AppSettings.AutostartFilePath;

        public void WriteFileToDisk()
        {
            if (_autostarts.Count > 0) Writer.WriteFile(_path, _autostarts, "autostart");
            else
            {
                if (System.IO.File.Exists(_path)) System.IO.File.Delete(_path);
            }
        }

        private void ReadFileFromDisk()
        {
            if (System.IO.File.Exists(_path))
            {
                ManagedAppCollection collection = new ManagedAppCollection();
                collection = (ManagedAppCollection)Reader.ReadFile(_path, collection, "autostart2");
                CopyToCollection(collection);
            }
        }


        private void SaveConfigExecute(object obj)
        {
            System.Windows.Forms.SaveFileDialog dialog = new System.Windows.Forms.SaveFileDialog()
            {
                Filter = AppSettings.AutostartFilter,
                AddExtension = true,
                InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory),
                Title = "Autostart-Konfiguration speichern"
            };
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Writer.WriteFile(dialog.FileName, _autostarts, "auto");
            }
        }

        private void RemoveSelectedExecute(object obj)
        {
            _autostarts.Remove((ManagedApplication)_autostartView.CurrentItem);
        }

        private void RemoveAllExecute(object obj)
        {
            _autostarts.Clear();
        }

        private void AddNewFolderExecute(object obj)
        {
            NLScC.dialog.AddFolderDialog dialog = new dialog.AddFolderDialog()
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if (dialog.ShowDialog() == true)
            {
                ManagedApplication app = new ManagedApplication(ApplicationType.Folder, dialog.FolderName.Text, dialog.FolderPath.Text);
                _autostarts.Add(app);
                AutostartView.MoveCurrentTo(app);
            }
        }

        private void AddNewWebExecute(object obj)
        {
            NLScC.dialog.AddWebDialog dialog = new dialog.AddWebDialog()
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if (dialog.ShowDialog() == true)
            {
                ManagedApplication app = new ManagedApplication(dialog.WebType, dialog.WebName.Text, dialog.WebURL.Text);
                _autostarts.Add(app);
                AutostartView.MoveCurrentTo(app);
            }
        }

        private void AddNewAppExecute(object obj)
        {
            NLScC.dialog.AddAppDialog dialog = new dialog.AddAppDialog(false, null)
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if (dialog.ShowDialog() == true)
            {
                ManagedApplication app = new ManagedApplication(ApplicationType.Application,dialog.AppName.Text, dialog.AppPath.Text, "", dialog.AppArgs.Text);
                _autostarts.Add(app);
                AutostartView.MoveCurrentTo(app);
            }
        }

        #endregion

        private void LoadTestData()
        {
            _autostarts.Add(new ManagedApplication(ApplicationType.Application, "Test 1,2,3", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Application, "Test 4,5,6", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Edge, "Edge", @"edge.de"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Folder, "Folder Test", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Firefox, "Firefo", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Website, "Website(tst)", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Chrome, "Chrome Testing", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Undefined, "Undefined", @"c:\temp"));
            _autostarts.Add(new ManagedApplication(ApplicationType.Application, "Neuer Test", @"c:\temp"));
        }

        private ManagedAppCollection _autostarts;
        private ListCollectionView _autostartView;
        public ListCollectionView AutostartView { get { return _autostartView; } }

        public AutostartViewModel()
        {
            InitializeCommands();
            _autostarts = new ManagedAppCollection();
            //LoadTestData();
            ReadFileFromDisk();
            _autostartView = new ListCollectionView(_autostarts);

            if(Environment.GetCommandLineArgs().Length > 1)
            {
                ExecuteStartUp();
            }
        }


        private async void ExecuteStartUp()
        {
            await Task.Delay(4000);
            await Task.Run(() => Execute());
        }

        private async void Execute()
        {
            if (AppSettings.Settings.ExecuteAutostartWithAppStart)
            {
                await Task.Delay(5000);
                ExecuteAllExecute(null);
            }
        }

    }
}
