﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;
using NLScC.model;
using NLScC.Command;
using System.Windows.Controls;

namespace NLScC.ViewModel
{
    public class MemberViewModel : NotifyBase
    {

        #region Calling Show


        private bool _callingShowAll;
        public bool CallingShowAll
        {
            get { return _callingShowAll; }
            set
            {
                SetField(ref _callingShowAll, value);
                CallingShowMeldestelle = value;
                CallingShowGasDispatching = value;
                CallingShowGasBackoffice = value;
                CallingShowMittelspannung = value;
                CallingShowHochspannung = value;
                CallingShowStromBackoffice = value;
                CallingShowSupervisor = value;
                CallingShowCustom = value;
            }
        }

        private bool _callingShowMeldestelle;
        public bool CallingShowMeldestelle
        {
            get { return _callingShowMeldestelle; }
            set
            {
                SetField(ref _callingShowMeldestelle, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowGasDispatching;
        public bool CallingShowGasDispatching
        {
            get { return _callingShowGasDispatching; }
            set
            {
                SetField(ref _callingShowGasDispatching, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowGasBackoffice;
        public bool CallingShowGasBackoffice
        {
            get { return _callingShowGasBackoffice; }
            set
            {
                SetField(ref _callingShowGasBackoffice, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowMittelspannung;
        public bool CallingShowMittelspannung
        {
            get { return _callingShowMittelspannung; }
            set
            {
                SetField(ref _callingShowMittelspannung, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowHochspannung;
        public bool CallingShowHochspannung
        {
            get { return _callingShowHochspannung; }
            set
            {
                SetField(ref _callingShowHochspannung, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowStromBackoffice;

        internal void WriteFileTodisk()
        {
            Writer.WriteFile(AppSettings.CustomFilePath, _customList,"memberdisk");
        }

        public bool CallingShowStromBackoffice
        {
            get { return _callingShowStromBackoffice; }
            set
            {
                SetField(ref _callingShowStromBackoffice, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowSupervisor;
        public bool CallingShowSupervisor
        {
            get { return _callingShowSupervisor; }
            set
            {
                SetField(ref _callingShowSupervisor, value);
                ApplyCallingFilter();
            }
        }
        private bool _callingShowCustom;
        public bool CallingShowCustom
        {
            get { return _callingShowCustom; }
            set
            {
                SetField(ref _callingShowCustom, value);
                ApplyCallingFilter();
            }
        }


        private void ApplyCallingFilter()
        {
            if (_memberView != null)
            {
                _memberView.Filter = Filtering;
            }
        }

        private bool Filtering(object obj)
        {
            switch((obj as NLSMember).Unit)
            {
                case NLSUnit.Meldestelle:
                    return CallingShowMeldestelle;
                case NLSUnit.Gasdispatching:
                    return CallingShowGasDispatching;
                case NLSUnit.GasBackoffice:
                    return CallingShowGasBackoffice;
                case NLSUnit.Mittelspannung:
                    return CallingShowMittelspannung;
                case NLSUnit.Hochspannung:
                    return CallingShowHochspannung;
                case NLSUnit.StromBackoffice:
                    return CallingShowStromBackoffice;
                case NLSUnit.Supervisor:
                    return CallingShowSupervisor;
                case NLSUnit.Custom:
                    return CallingShowCustom;
                default:
                    return false;
            }
        }

        #endregion

        #region Commands
        public ICommand SelectAllCommand { get; internal set; }
        public ICommand SelectVisibleCommand { get; internal set; }
        public ICommand SelectGroup { get; internal set; }
        public ICommand EditorCommand { get; internal set; }
        public ICommand PrepareMailCommand { get; internal set; }


        private void InitializeCommands()
        {
            SelectAllCommand = new RelayCommand(SelectAllExecute);
            SelectVisibleCommand = new RelayCommand(SelectVisibleExecute, SelectVisibleCanExecute);
            SelectGroup = new RelayCommand(SelectGroupExecute);
            EditorCommand = new RelayCommand(OpenEditor);
            PrepareMailCommand = new RelayCommand(PrepareMailExecute);
        }

        private void PrepareMailExecute(object obj)
        {
            ListView listview = MainWindow.AppReference.FindName("OutlookListView") as ListView;
            if (listview == null) return;
            System.Collections.IList items = listview.SelectedItems;
            var coll = items.Cast<NLSMember>();
            OutlookHelper.PrepareMailWithRecipients(coll);
        }

        private void OpenEditor(object obj)
        {
            NLScC.dialog.CustomDialog dialog = new dialog.CustomDialog(_nLSMembers)
            {
                FontSize = MainWindow.AppReference.FontSize,
                FontFamily = MainWindow.AppReference.FontFamily,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 4),
                Owner = MainWindow.AppReference
            };
            if (dialog.ShowDialog() == true)
            {
                FillCustomList(dialog.Collection);
            }
        }

        private void FillCustomList(NLSMemberCollection collection)
        {
            _customList.Clear();
            foreach (NLSMember member in collection)
            {
                _customList.Add(member);
            }
            MergeLists();
        }

        private void SelectGroupExecute(object obj)
        {
            NLSUnit unit;
            Enum.TryParse(Convert.ToString(obj), true,out unit);
            ListView listview = MainWindow.AppReference.FindName("OutlookListView") as ListView;
            foreach (NLSMember member in listview.Items)
            {
                if(member.Unit == unit && !listview.SelectedItems.Contains(member))
                {
                    listview.SelectedItems.Add(member);
                }
            }
        }

        private bool SelectVisibleCanExecute(object obj)
        {
            return MemberView.Count > 0;
        }

        private void SelectVisibleExecute(object obj)
        {
            ListView listview = MainWindow.AppReference.FindName("OutlookListView") as ListView;
            if (listview != null)
            {
                for (int i = 0; i < listview.Items.Count; i++)
                {
                    switch((listview.Items[i] as NLSMember).Unit)
                    {
                        case NLSUnit.Meldestelle:
                            if (CallingShowMeldestelle && !listview.SelectedItems.Contains(listview.Items[i]))listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.Gasdispatching:
                            if (CallingShowGasDispatching && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.GasBackoffice:
                            if (CallingShowGasBackoffice && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.Mittelspannung:
                            if (CallingShowMittelspannung && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.Hochspannung:
                            if (CallingShowHochspannung && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.StromBackoffice:
                            if (CallingShowStromBackoffice && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.Supervisor:
                            if (CallingShowSupervisor && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                        case NLSUnit.Custom:
                            if (CallingShowCustom && !listview.SelectedItems.Contains(listview.Items[i])) listview.SelectedItems.Add(listview.Items[i]);
                            break;
                    }
                }
            }
        }


        private void SelectAllExecute(object obj)
        {
            ListView listview = MainWindow.AppReference.FindName("OutlookListView") as ListView;
            if( listview != null)
            {
                if ((bool)obj)
                {
                    listview.SelectAll();
                }
                else
                {
                    listview.UnselectAll();
                }
            }
        }
        #endregion

        private NLSMemberCollection _memberList;
        private NLSMemberCollection _customList;

        private NLSMemberCollection _nLSMembers;
        private ListCollectionView _memberView;
        public ListCollectionView MemberView
        {
            get { return _memberView; }
        }


        public MemberViewModel()
        {
            CallingShowAll = true;
            InitializeCommands();
            LoadData();

            //LoadTestData();
            _memberView = new ListCollectionView(_nLSMembers);
            _memberView.GroupDescriptions.Add(new PropertyGroupDescription("Unit"));
            _memberView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Unit", System.ComponentModel.ListSortDirection.Ascending));
            _memberView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Lastname", System.ComponentModel.ListSortDirection.Ascending));
            _memberView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Prename", System.ComponentModel.ListSortDirection.Ascending));
        }


        private void LoadData()
        {
            if (System.IO.File.Exists(AppSettings.CustomFilePath))
            {
                _customList = (NLSMemberCollection)Reader.ReadFile(AppSettings.CustomFilePath, new NLSMemberCollection(), "member");
                LogConsole.Log("Custom-File erfolgreich gelesen");
            }
            if (System.IO.File.Exists(AppSettings.MemberFilePath))
            {
                _memberList = (NLSMemberCollection)Reader.ReadFile(AppSettings.MemberFilePath, new NLSMemberCollection(), "member2");
                LogConsole.Log("Member-File erfolgreich gelesen");
            }
            else
            {
                string message = "Die Member-File ist nicht lesbar. Dies hat eventuell Auswirkungen auf das Programm!!";
                LogConsole.LogLevel lvl = LogConsole.LogLevel.Warning;
                LogConsole.Log(message, lvl);
            }

            if (_customList == null) _customList = new NLSMemberCollection();
            if (_memberList == null) _memberList = new NLSMemberCollection();
            _nLSMembers = new NLSMemberCollection();
            MergeLists();

        }

        private void MergeLists()
        {
            _nLSMembers.Clear();
            foreach (NLSMember member in _customList)
            {
                _nLSMembers.Add(member);
            }
            foreach (NLSMember member in _memberList)
            {
                _nLSMembers.Add(member);
            }
            LogConsole.Log("NLS Members erfolgreich erstellt.");
        }

        private void LoadTestData()
        {
            _nLSMembers.Add(new NLSMember("Kamphenkel", "Pascal", "97517", "pascal.kamphenkel@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Blumenberg", "Jens", "97509", "jens.blumenberg@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Brandis", "Andreas", "97514", "andreas.brandis@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Günther", "Bennet", "XXXXX", "bennet.guenther@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Hain", "Alexander", "97518", "alexander.hain@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Jasbutis", "Michael", "97511", "michael.jasbutis@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Karlauf", "Ralf", "97510", "ralf.karlauf@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Poppe", "David", "XXXXX", "david.poppe@avacon.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Busche", "Felix", "97516", "felix.busche@avacon.de", NLSUnit.Gasdispatching));
            _nLSMembers.Add(new NLSMember("Czarnecki", "Andreas", "97500", "andreas.czarnecki@avacon.de", NLSUnit.Gasdispatching));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.GasBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.Mittelspannung));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.Mittelspannung));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.GasBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.Meldestelle));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
            _nLSMembers.Add(new NLSMember("Testname", "Testie", "30492", "testwerte@werte.de", NLSUnit.StromBackoffice));
        }
    }
}
