﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;
using NLScC.model;
using NLScC.Command;
using NLScC.dialog;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows;

namespace NLScC.ViewModel
{
    public class ProgramManagerViewModel
    {

        #region Commands

        public ICommand ProgramManagerAddApplication { get; internal set; }
        public ICommand ProgramManagerAddWebsite { get; internal set; }
        public ICommand ProgramManagerAddFolder { get; internal set; }
        public ICommand ProgramManagerReloadDictionary { get; internal set; }
        public ICommand ProgramManagerMoveToGroup { get; internal set; }
        public ICommand ProgramManagerEdit { get; internal set; }
        public ICommand ProgramManagerDeleteSelected { get; internal set; }
        public ICommand ProgramManagerDeleteAll { get; internal set; }
        public ICommand ProgramManagerSaveConfig { get; internal set; }
        public ICommand ProgramManagerLoadConfig { get; internal set; }
        public ICommand ProgramManagerDoubleClick { get; internal set; }
        public ICommand ProgramManagerRefresh { get; internal set; }
        public ICommand ProgramManagerClosEditor { get; internal set; }

        private void InitializeCommands()
        {
            ProgramManagerAddApplication = new RelayCommand(AddApplicationExecute);
            ProgramManagerAddWebsite = new RelayCommand(AddWebsiteExecute);
            ProgramManagerAddFolder = new RelayCommand(AddFolderExecute);
            ProgramManagerReloadDictionary = new RelayCommand(ReloadDictionaryExecute, CanOnlyExecuteWithEntries);
            ProgramManagerMoveToGroup = new RelayCommand(MoveGroupExecute, CanOnlyExecuteWithEntries);
            ProgramManagerEdit = new RelayCommand(EditExecute, CanOnlyExecuteWithEntries);
            ProgramManagerDeleteAll = new RelayCommand(DeleteAllExecute, CanOnlyExecuteWithEntries);
            ProgramManagerDeleteSelected = new RelayCommand(DeleteSelectedExecute, CanOnlyExecuteWithEntries);
            ProgramManagerSaveConfig = new RelayCommand(SaveExecute, CanOnlyExecuteWithEntries);
            ProgramManagerLoadConfig = new RelayCommand(LoadExecute);
            ProgramManagerDoubleClick = new RelayCommand(DoubleClick);
            ProgramManagerRefresh = new RelayCommand(Refresh);
            ProgramManagerClosEditor = new RelayCommand(CloseEditor);
        }

        private void CloseEditor(object obj)
        {
            Border border = MainWindow.AppReference.FindName("EditorWindow") as Border;
            ThicknessAnimation marginAnimation = new ThicknessAnimation
            {
                To = new Thickness(0, 0, 0, -1 * border.ActualHeight)
            };
            Duration duration = new Duration(TimeSpan.FromSeconds(0.5));
            marginAnimation.Duration = duration;
            border.BeginAnimation(Border.MarginProperty, marginAnimation);
        }

        private void Refresh(object obj)
        {
            ProgramManagerView.Refresh();
        }

        private void DoubleClick(object obj)
        {
            var parameter = (Tuple<ApplicationType, string, string>)obj;
            switch (parameter.Item1)
            {
                case ApplicationType.Application:
                    Executioner.ExecuteApp(parameter.Item2, parameter.Item3);
                    break;
                case ApplicationType.Chrome:
                case ApplicationType.Edge:
                case ApplicationType.Firefox:
                case ApplicationType.Website:
                    Executioner.ExecuteWebsite(parameter.Item2, parameter.Item1);
                    break;
                case ApplicationType.Folder:
                    Executioner.ExecuteFolder(parameter.Item2, parameter.Item3);
                    break;
                default:
                    string message = "Öffnen des angegebenen Eintrags {path->" + parameter.Item2 + "} nicht möglich. Fehler in {ApplicationType->Unknown}";
                    LogConsole.Log(message, LogConsole.LogLevel.Warning);
                    break;
            }
        }

        private bool CanOnlyExecuteWithEntries(object obj)
        {
            return _managedApplications.Count > 0;
        }

        private void LoadExecute(object obj)
        {
            System.Windows.Forms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog()
            {
                Filter = AppSettings.ProgramManagerFilter,
                AddExtension = true,
                InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory),
                Title = "Autostart-Konfiguration laden"
            };
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                ManagedAppCollection o = (Reader.ReadFile(dialog.FileName, _managedApplications, "programm") as ManagedAppCollection);
                NLScC.dialog.OverrideDialog.OverrideMode mode = NLScC.dialog.OverrideDialog.OverrideMode.Override;
                if (o.Count > 0 && _managedApplications.Count > 0)
                {
                    NLScC.dialog.OverrideDialog overrideDialog = new dialog.OverrideDialog()
                    {
                        Top = MainWindow.AppReference.Top + 10,
                        Left = MainWindow.AppReference.Left + (MainWindow.AppReference.Width / 2),
                        Owner = MainWindow.AppReference
                    };
                    if (overrideDialog.ShowDialog() == true)
                    {
                        mode = overrideDialog.OverrideResult;
                    }
                }
                if (mode == NLScC.dialog.OverrideDialog.OverrideMode.Override) _managedApplications.Clear();
                LogConsole.Log("Konfigruation wird geladen -> OverrideMode." + mode);
                CopyToCollection(o);
            }
        }

        private void SaveExecute(object obj)
        {
            System.Windows.Forms.SaveFileDialog dialog = new System.Windows.Forms.SaveFileDialog()
            {
                Filter = AppSettings.ProgramManagerFilter,
                AddExtension = true,
                InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory),
                Title = "Autostart-Konfiguration speichern"
            };
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Writer.WriteFile(dialog.FileName, _managedApplications, "programm2");
            }
        }

        private void DeleteSelectedExecute(object obj)
        {
            _managedApplications.Remove((ManagedApplication)_programManagerView.CurrentItem);
        }

        private void DeleteAllExecute(object obj)
        {
            var s = MainWindow.AppReference;
            DeleteAllSureDialog dialog = new DeleteAllSureDialog()
            {
                FontSize = s.FontSize,
                Left = s.Left + (s.ActualWidth / 2),
                Top = s.Top + 10
            };

            if (dialog.ShowDialog() == true)
            {
                _managedApplications.Clear();
            }
        }

        private void EditExecute(object obj)
        {
           Border border = MainWindow.AppReference.FindName("EditorWindow") as Border;
            if (border != null)
            {
                ThicknessAnimation thicknessAnimation = new ThicknessAnimation
                {
                    To = new Thickness(0, 0, 0, 0),
                    DecelerationRatio = 0.988
                };
                thicknessAnimation.Duration = new Duration(TimeSpan.FromSeconds(.5));
                border.BeginAnimation(Border.MarginProperty, thicknessAnimation);
            }
        }

        private void MoveGroupExecute(object obj)
        {
            throw new NotImplementedException();
        }

        private void ReloadDictionaryExecute(object obj)
        {
            throw new NotImplementedException();
        }

        private void AddFolderExecute(object obj)
        {
            AddFolderDialog dialog = new AddFolderDialog(true, GetGroupList()) 
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if(dialog.ShowDialog() == true)
            {
                _managedApplications.Add(new ManagedApplication(ApplicationType.Folder, dialog.FolderName.Text, dialog.FolderPath.Text, dialog.GroupInsert.Text, expanded: true));
            }
        }

        private void AddWebsiteExecute(object obj)
        {
            AddWebDialog dialog = new AddWebDialog(true, GetGroupList())
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if ( dialog.ShowDialog() == true)
            {
                _managedApplications.Add(new ManagedApplication(dialog.WebType, dialog.WebName.Text, dialog.WebURL.Text, dialog.GroupInsert.Text, expanded: true));
            }
        }

        private void AddApplicationExecute(object obj)
        {
            AddAppDialog dialog = new AddAppDialog(true, GetGroupList())
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + (MainWindow.AppReference.ActualWidth / 2)
            };
            if (dialog.ShowDialog() == true)
            {
                _managedApplications.Add(new ManagedApplication(ApplicationType.Application, dialog.AppName.Text, dialog.AppPath.Text, dialog.GroupInsert.Text, dialog.AppArgs.Text,expanded: true));
            }
        }

        private List<string> GetGroupList()
        {
            List<string> list = new List<string>();
            foreach(ManagedApplication managed in _managedApplications)
            {
                if (!list.Contains(managed.Group)) list.Add(managed.Group);
            }
            return list;
        }



        #endregion

        private ManagedAppCollection _managedApplications;
        private ListCollectionView _programManagerView;
        public ListCollectionView ProgramManagerView { get { return _programManagerView; } }


        public ProgramManagerViewModel()
        {
            InitializeCommands();
            _managedApplications = new ManagedAppCollection();
            //LoadFilesFromDisk();
            LoadTestData();
           
            _programManagerView = new ListCollectionView(_managedApplications);
            //_programManagerView.GroupDescriptions.Add(new PropertyGroupDescription("Group"));
        }

        public void WriteFilesToDisk()
        {
            if(_managedApplications.Count > 0)   Writer.WriteFile(AppSettings.ProgramManagerFilePath, _managedApplications,"pm");
            else
            {
                if (System.IO.File.Exists(AppSettings.ProgramManagerFilePath)) System.IO.File.Delete(AppSettings.ProgramManagerFilePath);
            }
        }

        private void LoadFilesFromDisk()
        {
            string path = AppSettings.ProgramManagerFilePath;
            if (System.IO.File.Exists(path))
            {
                ManagedAppCollection appColl = new ManagedAppCollection();
                appColl = (ManagedAppCollection)Reader.ReadFile(path, appColl,"pmm");
                CopyToCollection(appColl);
                LogConsole.Log("PgrogramManager-Files erfolgreich geladen");
            }
        }

        private void CopyToCollection(ManagedAppCollection collection)
        {
            foreach (ManagedApplication ma in collection)
            {
                _managedApplications.Add(ma);
            }
            LogConsole.Log("Konfiguration erfolgreich geladen");
        }


        private void LoadTestData()
        {
            int k = 0;
            for (int i = 0; i < 70; i++)
            {
                _managedApplications.Add(new ManagedApplication(ApplicationType.Application, "Test" + i, @"C:\Users\P16250\Desktop\NLSCC2Test\mappe1.xlsx", k.ToString(), "", "", true));
                _managedApplications.Add(new ManagedApplication(ApplicationType.Application, "Test_" + i, @"C:\Programmierung\GSFDAT\gsfdat.cs", k.ToString(), "", "", true));
                _managedApplications.Add(new ManagedApplication(ApplicationType.Application, "Test_A" + i, @"C:\Users\P16250\Videos\wildlife.wmv", k.ToString(), "", "", true));
                _managedApplications.Add(new ManagedApplication(ApplicationType.Application, "Test_BBA" + i, @"C:\Users\P16250\Desktop\start.bat", k.ToString(), "", "", true));

                if (i % 10 == 0) k++;
            }
        }
    }
}
