﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Xml.Serialization;
using System.Xml;
using System.Runtime.Serialization;
using NLScC.ItemSources;
using System.Windows;

namespace NLScC.model
{
    [DataContract]
    public class AppSettings : NotifyBase
    {

        public static AppSettings Settings
        {
            get;
            private set;
        }

        static AppSettings()
        {
            Settings = new AppSettings()
            {
                AppHeight = 450,
                AppWidth = 650,
                LastUpdate = DateTime.Today,
                FontSize = 12,
                FontFamily = FontArt.Calibri,
                Foreground = "Black",
                ButtonSize = 30,
                TabSize = 30,
                IsStartetWithWindows = false,
                ExecuteAutostartWithAppStart = false,
                ManagedAppOffset = 4,
                ManagedAppHeight = 4,
                ManagedAppWidth = 4,
                ManagedAppImageAlignment = 2,
                _managedAppImageSize = 4,
                _managedAppFontSize = 16
            };
        }


        public static void LoadExistingConfig(AppSettings s)
        {
            Settings = s;
            LoadConfigFromDisk();
        }




        private string _name = "NLS Control Center - ";
        private string _version = "2.0.0.0";

        public string LongVersion { get { return _name + _version; } }
        [DataMember]
        public string Version { get { return _version; } set { SetField(ref _version, value); } }




        public void SaveConfigToDisk()
        {
            string workingDirectory = _roamingDirectory; ;
            string file = workingDirectory + @"\settings.nsave";
            var ser = new DataContractSerializer(this.GetType());
            using (XmlWriter writer = XmlWriter.Create(file))
            {
                ser.WriteObject(writer, this);
            }
        }

        public static void LoadConfigFromDisk()
        {
            string workingDirectory = _roamingDirectory;
            string file = workingDirectory + @"\settings.nsave";
            AppSettings settings;
            if (System.IO.File.Exists(file))
            {
                using (XmlReader reader = XmlReader.Create(file))
                {
                    var serializer = new DataContractSerializer(typeof(AppSettings));
                    settings = (AppSettings)serializer.ReadObject(reader);
                }
            }
            else
            {
                settings = new AppSettings()
                {
                    AppHeight = 450,
                    AppWidth = 650,
                    LastUpdate = DateTime.Today,
                    FontSize = 12,
                    FontFamily = FontArt.Calibri,
                    Foreground = "Black",
                    ButtonSize = 30,
                    TabSize = 30,
                    IsStartetWithWindows = false,
                    ExecuteAutostartWithAppStart = false,
                    ManagedAppOffset = 4,
                    ManagedAppHeight = 4,
                    ManagedAppWidth = 4,
                    ManagedAppImageAlignment = 2,
                    _managedAppImageSize = 4
                };
            }
            OverrrideLoadedData(settings);
        }

        public static void OverrrideLoadedData(AppSettings s)
        {
            Settings._appMultiMonitorWidth = s._appMultiMonitorWidth;
            Settings._appMultiMonitorHeight = s._appMultiMonitorHeight;
            Settings._appMultiMonitorLeft = s._appMultiMonitorLeft;
            Settings._appMultiMonitorTop = s._appMultiMonitorTop;
            Settings._appHeight = s._appHeight;
            Settings._appWidth = s._appWidth;
            Settings._appLeft = s._appLeft;
            Settings._appTop = s._appTop;
            Settings.LastUpdate = s._lastUpdate;
            Settings.FontFamily = s._fontFamily;
            Settings.FontSize = s._fontSize;
            Settings.Foreground = s._foreground;
            Settings.ButtonSize = s._buttonSize;
            Settings.TabSize = s._tabSize;
            Settings.IsStartetWithWindows = s._isStartetWithWindows;
            Settings.ExecuteAutostartWithAppStart = s._executeAutostartWithAppStart;

            Settings.ManagedAppFontSize = s._managedAppFontSize;
            Settings.ManagedAppHeight = s._managedAppHeight;
            Settings.ManagedAppImageSize = s._managedAppImageSize;
            Settings.ManagedAppOffset = s._managedAppOffset;
            Settings.ManagedAppWidth = s._managedAppWidth;
            Settings.ManagedAppImageAlignment = s.ManagedAppImageAlignment;
        }

        [DataMember(Name = "AppWidth")]
        private int _appWidth;
        [DataMember(Name = "AppWidthMulti")]
        private int _appMultiMonitorWidth;
        public int AppWidth { get { return GetWidth(); } set { SetWidth(value); } }

        [DataMember(Name = "AppHeight")]
        private int _appHeight;
        [DataMember(Name = "AppHeightMulti")]
        private int _appMultiMonitorHeight;
        public int AppHeight { get { return GetHeight(); } set { SetHeight(value); } }

        [DataMember(Name = "AppLeft")]
        private int _appLeft;
        [DataMember(Name = "AppLeftMulti")]
        private int _appMultiMonitorLeft;
        public int AppLeft { get { return GetLeft(); ; } set { SetLeft(value); } }

        [DataMember(Name = "AppTop")]
        private int _appTop;
        [DataMember(Name = "AppTopMulti")]
        private int _appMultiMonitorTop;
        public int AppTop { get { return GetTop(); } set { SetTop(value); } }

        [DataMember]
        private bool _isMultiMonitor;
        public bool IsMultiMonitor { get { return _isMultiMonitor; } set { SetField(ref _isMultiMonitor, value); } }



        private void SetWidth(int value)
        {
            if (IsMultiMonitor)
            {
                SetField(ref _appMultiMonitorWidth, value);
            }
            else
            {
                SetField(ref _appWidth, value);
            }
        }

        private int GetWidth()
        {
            if (IsMultiMonitor) return _appMultiMonitorWidth;
            return _appWidth;
        }

        private void SetHeight(int value)
        {
            if (IsMultiMonitor)
            {
                SetField(ref _appMultiMonitorHeight, value);
            }
            else
            {
                SetField(ref _appHeight, value);
            }
        }

        private int GetHeight()
        {
            if (IsMultiMonitor) return _appMultiMonitorHeight;
            return _appHeight;
        }

        private void SetLeft(int value)
        {
            if (IsMultiMonitor)
            {
                SetField(ref _appMultiMonitorLeft, value);
            }
            else
            {
                SetField(ref _appLeft, value);
            }
        }

        private int GetLeft()
        {
            if (IsMultiMonitor) return _appMultiMonitorLeft;
            return _appLeft;
        }

        private void SetTop(int value)
        {
            if (IsMultiMonitor)
            {
                SetField(ref _appMultiMonitorTop, value);
            }
            else
            {
                SetField(ref _appTop, value);
            }
        }

        private int GetTop()
        {
            if (IsMultiMonitor) return _appMultiMonitorTop;
            return _appTop;
        }


        private DateTime _lastUpdate;
        [DataMember]
        public DateTime LastUpdate { get { return _lastUpdate; } set { SetField(ref _lastUpdate, value); } }

        public static string TeamDirectory = @"G:\TEAM\EAV\V-N\NT\NTS\NTS-R\05_Kollektiv\Programmdateien\NLSCC";

        private static string _roamingDirectory = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "NLScC");
        public static string RoamingDirectory { get { return _roamingDirectory; } }

        private static string _favIconPath = System.IO.Path.Combine(_roamingDirectory, "FavIcons");
        public static string FavIconPath { get { return _favIconPath; } }

        private static string _dataPath = System.IO.Path.Combine(_roamingDirectory, "Data");
        public static string DataPath { get { return _dataPath; } }

        private static string _customFilePath = System.IO.Path.Combine(_dataPath, @"custom.nlscco");
        public static string CustomFilePath {  get { return _customFilePath; } }

        private static string _memberFilePath = System.IO.Path.Combine(_dataPath, @"members.nlscco");
        public static string MemberFilePath { get { return _memberFilePath; } }

        private static string _programmanagerFilePath = System.IO.Path.Combine(_dataPath, @"programmanager.nlsccp");
        public static string ProgramManagerFilePath { get { return _programmanagerFilePath; } }

        private static string _autostartFilePath = System.IO.Path.Combine(_dataPath, @"autostart.nlsccp");
        public static string AutostartFilePath { get { return _autostartFilePath; } }

        private static string _phonebookFilePath = System.IO.Path.Combine(_dataPath, @"phonebook.nlscc");
        public static string PhonebookFilePath { get { return _phonebookFilePath; } }

        private static string _phonebookTeamPath = System.IO.Path.Combine(TeamDirectory, @"phonebook.nlscc");
        public static string PhonebookTeamPath { get { return _phonebookTeamPath; } }

        private static string _memberTeamPath = System.IO.Path.Combine(TeamDirectory, @"members.nlscc");
        public static string MemberTeamPath { get { return _memberTeamPath; } }




        private int _fontSize = 12;
        [DataMember]
        public int FontSize { get { return _fontSize; } set { SetField(ref _fontSize, value); CalculateFontSize(); } }

        private FontArt _fontFamily;
        [DataMember]
        public FontArt FontFamily { get { return _fontFamily; } set { SetField(ref _fontFamily, value); } }

        private string _foreground;
        [DataMember]
        public string Foreground { get { return _foreground; } set { SetField(ref _foreground, value); } }

        private byte _buttonSize;
        [DataMember]
        public byte ButtonSize { get { return _buttonSize; } set { SetField(ref _buttonSize, value); } }

        private byte _tabSize;
        [DataMember]
        public byte TabSize { get { return _tabSize; } set { SetField(ref _tabSize, value); } }

        private bool _isStartetWithWindows;
        [DataMember]
        public bool IsStartetWithWindows { get { return _isStartetWithWindows; } set { SetField(ref _isStartetWithWindows, value); } }

        private bool _executeAutostartWithAppStart;
        [DataMember]
        public bool ExecuteAutostartWithAppStart { get { return _executeAutostartWithAppStart; } set { SetField(ref _executeAutostartWithAppStart, value); } }


        //MangedApp Settings

        private byte _managedAppWidth;
        [DataMember]
        public byte ManagedAppWidth { get { return _managedAppWidth; } set { SetField(ref _managedAppWidth, value); } }

        private byte _managedAppHeight;
        [DataMember]
        public byte ManagedAppHeight { get { return _managedAppHeight; } set { SetField(ref _managedAppHeight, value); } }

        private byte _managedAppFontSize;
        [DataMember]
        public byte ManagedAppFontSize { get { return _managedAppFontSize; } private set { SetField(ref _managedAppFontSize, value); } }

        private byte _managedAppOffset;
        public byte ManagedAppOffset { get { return _managedAppOffset; } set { SetField(ref _managedAppOffset, value); CalculateFontSize(); } }

        private byte _managedAppImageSize;
        [DataMember]
        public byte ManagedAppImageSize { get { return _managedAppImageSize; } set { SetField(ref _managedAppImageSize, value); } }


        private byte _managedImageAlignment;
        [DataMember]
        public byte ManagedAppImageAlignment { get { return _managedImageAlignment; } set { SetField(ref _managedImageAlignment, value); } }


        private void CalculateFontSize()
        {
            byte b = (byte)(Convert.ToByte(ManagedAppOffset) + Convert.ToByte(FontSize) - 4);
            SetField<byte>(field: ref _managedAppFontSize, value: (b), _managedAppFontSize.ToString());
        }
        //



        private static string _autostartFilter = "NLSCC|*.nlscca";
        public static string AutostartFilter { get { return _autostartFilter; } }
        private static string _programManagerFilter = "NLSCC|*.nlsccp";
        public static string ProgramManagerFilter { get { return _programManagerFilter; } }

    }
}
