﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace NLScC.model
{
    public static class Executioner
    {

        public static void ExecuteApp(string path, string args)
        {
            Task.Run(() => StartApplication(path, args));
        }

        public static void ExecuteWebsite(string URL, ApplicationType WebTyp)
        {
            Task.Run(() => StartWebbrowser(URL, WebTyp));
        }

        public static void ExecuteFolder(string directory, string args)
        {
            Task.Run(() => StartFolder(directory));
        }


        private static void StartApplication(string path, string args)
        {
            try
            {
                System.Diagnostics.Process process = System.Diagnostics.Process.Start(path, args);
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Program <" + path + "> erfolgreich gestartet"));
            }
            catch (System.IO.FileNotFoundException)
            {
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Das angegebene Programm  <" + path + "> konnte nicht gestartet werden", LogConsole.LogLevel.Warning));
            }
            catch(System.ComponentModel.Win32Exception E)
            {
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Das angegebene Programm  <" + path + "> konnte nicht gestartet werden. Eine Win32Exception wurde ausgelöst: " + E.Message, LogConsole.LogLevel.Error));
            }
            catch(Exception e)
            {
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Das angegebene Programm  <" + path + "> konnte nicht gestartet werden. Exception: " + e.Message, LogConsole.LogLevel.Error));
            }
        }

        private static void StartWebbrowser(string path, ApplicationType type)
        {
            try
            {
                if (!path.StartsWith("http")) path = "http://" + path;
                switch (type)
                {
                    case ApplicationType.Chrome:
                        System.Diagnostics.Process chrome = System.Diagnostics.Process.Start("chrome.exe", path);
                        break;
                    case ApplicationType.Firefox:
                        System.Diagnostics.Process firefox = System.Diagnostics.Process.Start("firefox.exe", path);
                        break;
                    case ApplicationType.Edge:
                        System.Diagnostics.Process edge = System.Diagnostics.Process.Start("microsoft-edge:" + path);
                        break;
                    case ApplicationType.Website:
                        if (!path.StartsWith("http")) path = "http://" + path;
                        System.Diagnostics.Process process = System.Diagnostics.Process.Start(path);
                        break;
                }
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log(type.ToString() + " geöffnet: <" + path + ">"));
            }
            catch (Exception )
            {
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Fehler beim öffnen der Website: <" + path + "> und dem Attribut: <" + type + "> - Solche Fehler bitte melden!", LogConsole.LogLevel.Warning));
            }
        }

        private static void StartFolder(string path)
        {
            try
            {
                System.Diagnostics.Process process = System.Diagnostics.Process.Start("explorer.exe", path);
            }
            catch (Exception)
            {
                LogConsole.Logger.Dispatcher.Invoke(() => LogConsole.Log("Explorer geöffnet: <" + path + ">"));
            }
        }
    }
}
